@@ -4,7 +4,7 @@ use std::path::Path;
 use std::str::FromStr;
 
 use thiserror::Error;
-
+use uv_cache_key::{CacheKey, CacheKeyHasher};
 use uv_distribution_filename::DistExtension;
 use uv_fs::{CWD, PortablePath, PortablePathBuf, relative_to};
 use uv_git_types::{GitOid, GitReference, GitUrl, GitUrlParseError, OidParseError};
@@ -195,7 +195,7 @@ impl PartialOrd for Requirement {
 impl From<Requirement> for uv_pep508::Requirement<VerbatimUrl> {
     /// Convert a [`Requirement`] to a [`uv_pep508::Requirement`].
     fn from(requirement: Requirement) -> Self {
-        uv_pep508::Requirement {
+        Self {
             name: requirement.name,
             extras: requirement.extras,
             marker: requirement.marker,
@@ -216,7 +216,7 @@ impl From<Requirement> for uv_pep508::Requirement<VerbatimUrl> {
 impl From<Requirement> for uv_pep508::Requirement<VerbatimParsedUrl> {
     /// Convert a [`Requirement`] to a [`uv_pep508::Requirement`].
     fn from(requirement: Requirement) -> Self {
-        uv_pep508::Requirement {
+        Self {
             name: requirement.name,
             extras: requirement.extras,
             marker: requirement.marker,
@@ -299,7 +299,7 @@ impl From<uv_pep508::Requirement<VerbatimParsedUrl>> for Requirement {
                 RequirementSource::from_parsed_url(url.parsed_url, url.verbatim)
             }
         };
-        Requirement {
+        Self {
             name: requirement.name,
             groups: Box::new([]),
             extras: requirement.extras,
@@ -365,6 +365,107 @@ impl Display for Requirement {
     }
 }
 
+impl CacheKey for Requirement {
+    fn cache_key(&self, state: &mut CacheKeyHasher) {
+        self.name.as_str().cache_key(state);
+
+        self.groups.len().cache_key(state);
+        for group in &self.groups {
+            group.as_str().cache_key(state);
+        }
+
+        self.extras.len().cache_key(state);
+        for extra in &self.extras {
+            extra.as_str().cache_key(state);
+        }
+
+        if let Some(marker) = self.marker.contents() {
+            1u8.cache_key(state);
+            marker.to_string().cache_key(state);
+        } else {
+            0u8.cache_key(state);
+        }
+
+        match &self.source {
+            RequirementSource::Registry {
+                specifier,
+                index,
+                conflict: _,
+            } => {
+                0u8.cache_key(state);
+                specifier.len().cache_key(state);
+                for spec in specifier.iter() {
+                    spec.operator().as_str().cache_key(state);
+                    spec.version().cache_key(state);
+                }
+                if let Some(index) = index {
+                    1u8.cache_key(state);
+                    index.url.cache_key(state);
+                } else {
+                    0u8.cache_key(state);
+                }
+                // `conflict` is intentionally omitted
+            }
+            RequirementSource::Url {
+                location,
+                subdirectory,
+                ext,
+                url,
+            } => {
+                1u8.cache_key(state);
+                location.cache_key(state);
+                if let Some(subdirectory) = subdirectory {
+                    1u8.cache_key(state);
+                    subdirectory.display().to_string().cache_key(state);
+                } else {
+                    0u8.cache_key(state);
+                }
+                ext.name().cache_key(state);
+                url.cache_key(state);
+            }
+            RequirementSource::Git {
+                git,
+                subdirectory,
+                url,
+            } => {
+                2u8.cache_key(state);
+                git.to_string().cache_key(state);
+                if let Some(subdirectory) = subdirectory {
+                    1u8.cache_key(state);
+                    subdirectory.display().to_string().cache_key(state);
+                } else {
+                    0u8.cache_key(state);
+                }
+                url.cache_key(state);
+            }
+            RequirementSource::Path {
+                install_path,
+                ext,
+                url,
+            } => {
+                3u8.cache_key(state);
+                install_path.cache_key(state);
+                ext.name().cache_key(state);
+                url.cache_key(state);
+            }
+            RequirementSource::Directory {
+                install_path,
+                editable,
+                r#virtual,
+                url,
+            } => {
+                4u8.cache_key(state);
+                install_path.cache_key(state);
+                editable.cache_key(state);
+                r#virtual.cache_key(state);
+                url.cache_key(state);
+            }
+        }
+
+        // `origin` is intentionally omitted
+    }
+}
+
 /// The different locations with can install a distribution from: Version specifier (from an index),
 /// HTTP(S) URL, git repository, and path.
 ///
@@ -443,23 +544,23 @@ impl RequirementSource {
     /// the PEP 508 string (after the `@`) as [`VerbatimUrl`].
     pub fn from_parsed_url(parsed_url: ParsedUrl, url: VerbatimUrl) -> Self {
         match parsed_url {
-            ParsedUrl::Path(local_file) => RequirementSource::Path {
+            ParsedUrl::Path(local_file) => Self::Path {
                 install_path: local_file.install_path.clone(),
                 ext: local_file.ext,
                 url,
             },
-            ParsedUrl::Directory(directory) => RequirementSource::Directory {
+            ParsedUrl::Directory(directory) => Self::Directory {
                 install_path: directory.install_path.clone(),
                 editable: directory.editable,
                 r#virtual: directory.r#virtual,
                 url,
             },
-            ParsedUrl::Git(git) => RequirementSource::Git {
+            ParsedUrl::Git(git) => Self::Git {
                 git: git.url.clone(),
                 url,
                 subdirectory: git.subdirectory,
             },
-            ParsedUrl::Archive(archive) => RequirementSource::Url {
+            ParsedUrl::Archive(archive) => Self::Url {
                 url,
                 location: archive.url,
                 subdirectory: archive.subdirectory,
@@ -470,7 +571,7 @@ impl RequirementSource {
 
     /// Convert the source to a [`VerbatimParsedUrl`], if it's a URL source.
     pub fn to_verbatim_parsed_url(&self) -> Option<VerbatimParsedUrl> {
-        match &self {
+        match self {
             Self::Registry { .. } => None,
             Self::Url {
                 location,
@@ -567,21 +668,18 @@ impl RequirementSource {
     /// If the source is the registry, return the version specifiers
     pub fn version_specifiers(&self) -> Option<&VersionSpecifiers> {
         match self {
-            RequirementSource::Registry { specifier, .. } => Some(specifier),
-            RequirementSource::Url { .. }
-            | RequirementSource::Git { .. }
-            | RequirementSource::Path { .. }
-            | RequirementSource::Directory { .. } => None,
+            Self::Registry { specifier, .. } => Some(specifier),
+            Self::Url { .. } | Self::Git { .. } | Self::Path { .. } | Self::Directory { .. } => {
+                None
+            }
         }
     }
 
     /// Convert the source to a [`RequirementSource`] relative to the given path.
     pub fn relative_to(self, path: &Path) -> Result<Self, io::Error> {
         match self {
-            RequirementSource::Registry { .. }
-            | RequirementSource::Url { .. }
-            | RequirementSource::Git { .. } => Ok(self),
-            RequirementSource::Path {
+            Self::Registry { .. } | Self::Url { .. } | Self::Git { .. } => Ok(self),
+            Self::Path {
                 install_path,
                 ext,
                 url,
@@ -592,7 +690,7 @@ impl RequirementSource {
                 ext,
                 url,
             }),
-            RequirementSource::Directory {
+            Self::Directory {
                 install_path,
                 editable,
                 r#virtual,
@@ -613,10 +711,8 @@ impl RequirementSource {
     #[must_use]
     pub fn to_absolute(self, root: &Path) -> Self {
         match self {
-            RequirementSource::Registry { .. }
-            | RequirementSource::Url { .. }
-            | RequirementSource::Git { .. } => self,
-            RequirementSource::Path {
+            Self::Registry { .. } | Self::Url { .. } | Self::Git { .. } => self,
+            Self::Path {
                 install_path,
                 ext,
                 url,
@@ -625,7 +721,7 @@ impl RequirementSource {
                 ext,
                 url,
             },
-            RequirementSource::Directory {
+            Self::Directory {
                 install_path,
                 editable,
                 r#virtual,
@@ -819,7 +915,7 @@ impl From<RequirementSource> for RequirementSourceWire {
 impl TryFrom<RequirementSourceWire> for RequirementSource {
     type Error = RequirementError;
 
-    fn try_from(wire: RequirementSourceWire) -> Result<RequirementSource, RequirementError> {
+    fn try_from(wire: RequirementSourceWire) -> Result<Self, RequirementError> {
         match wire {
             RequirementSourceWire::Registry {
                 specifier,
